/*
*********************************************************************************************************
*                                               uC/OS-II
*                                        The Real-Time Kernel
*
*                          (c) Copyright 1992-1999, Jean J. Labrosse, Weston, FL
*                                          All Rights Reserved
*
*                                      INFINEON C167CR-LM Specific code
*                                          HLARGE MEMORY MODEL
*
* File : OS_CPU.H
* By   : Jean J. Labrosse
* By   : Michael Awosika
*********************************************************************************************************
*/
#ifndef UCOS_II
#define UCOS_II  1

/*
*********************************************************************************************************
*                                           DATA TYPES
*********************************************************************************************************
*/

#ifdef  OS_CPU_GLOBALS
#define OS_CPU_EXT
#else
#define OS_CPU_EXT  extern
#endif

/*
*********************************************************************************************************
*                                              DATA TYPES
*                                         (Compiler Specific)
*********************************************************************************************************
*/

typedef unsigned char  BOOLEAN;
typedef unsigned char  INT8U;                    /* Unsigned  8 bit quantity                           */
typedef signed   char  INT8S;                    /* Signed    8 bit quantity                           */
typedef unsigned int   INT16U;                   /* Unsigned 16 bit quantity                           */
typedef signed   int   INT16S;                   /* Signed   16 bit quantity                           */
typedef unsigned long  INT32U;                   /* Unsigned 32 bit quantity                           */
typedef signed   long  INT32S;                   /* Signed   32 bit quantity                           */
typedef float          FP32;                     /* Single precision floating point                    */
typedef double         FP64;                     /* Double precision floating point                    */

typedef unsigned int   OS_STK;                   /* Each stack entry is 16-bit wide                    */

#define BYTE           INT8S                     /* Define data types for backward compatibility ...   */
#define UBYTE          INT8U                     /* ... to uC/OS V1.xx.  Not actually needed for ...   */
#define WORD           INT16S                    /* ... uC/OS-II.                                      */
#define UWORD          INT16U
#define LONG           INT32S
#define ULONG          INT32U

/*
*********************************************************************************************************
*                                     Infineon C16x (Large Model)
*
* Method #1:  Disable/Enable interrupts using simple instructions.  After critical section, interrupts
*             will be enabled even if they were disabled before entering the critical section.  You MUST
*             change the constant in OS_CPU_A.ASM, function OSIntCtxSw() from 10 to 8.
* Method #2:  Not supported.
*********************************************************************************************************
*/

#define  OS_CRITICAL_METHOD 1
#define  OS_EXIT_CRITICAL()    (IEN = 1)              /* Enable  interrupts                            */
#define  OS_ENTER_CRITICAL()   (IEN = 0)              /* Disable interrupts                            */

/*
*********************************************************************************************************
*                             Infineon C16x (Large Model) Miscellaneous
*********************************************************************************************************
*/

#define  OS_STK_GROWTH        1                       /* Stack grows from HIGH to LOW memory on C16x   */

#define  uCOS                 0x50                    /* Interrupt vector 0x50 used for context switch */
#define  uCOSTMR              0x20                    /* Interrupt vector 0x20 used for context switch */

#define  OS_TASK_SW()         _trap_(uCOS)           /* Software trap                                  */
#define  OS_TASK_ISR()        _trap_(uCOSTMR)        /* Software trap for timer                        */


/*
*********************************************************************************************************
*                                         FUNCTION PROTOTYPES
*********************************************************************************************************
*/

void OSTaskBuildStk (INT16U page, INT16U offset, INT16U datapag, INT16U datapof);
void OSTickISRInit(void);

#endif


