// History-----------------------------------------------------------------------------------------------------------------
/*
V 1.0 von Juni 05, Grundversion, ist schon frher entstanden
V 1.1 vom 13.11.05
um einstellbare Bitrate und wartende Sende- und Empfangsfunktionen erweitert
paket_ready Verarbeitung entfernt
*/
// Resourcen-----------------------------------------------------------------------------------------------------------------
/*
P3.8 Receive von COM
P3.9 Transmit an COM
P8.0 weitergeleitetes Receive Signal

Da das Receive Signal auf Evalboard Hardware an Port 3.8 reinkommt und dort aber keine CAPCOM Unit dahinter liegt
wird das Signal an Port 8.0 per Drahtbrcke weitergeleitet.
Drahtbrcke von P3.8 zu P8.0, von links unten gezhlt 6 freilassen, einstecken, drei freilassen, einstecken

Fr Jumpering des Evalboards auf RS232-Betrieb siehe eigenes Dokument "Evalboard Jumper Settings"

CC16 hinter P8.0
T7, mit hchster Geschwindigkeit initialisiert (400 ns tick)
CC24 als Timer fr Senden


Interrupts:

C16IC=0x004C;	// CC16 fr receive_isr, IE=1 ILVL=0011  GLVL=00, Int-Nr. 0x30
C24IC=0x004D;	// CC24 fr send_isr, IE=1 ILVL=0011  GLVL=01, Int-Nr. 0x38

*/
// Includes ----------------------------------------------------------

#include <reg167.h>
#include <string.h>


// Defines ----------------------------------------------------------

#define MAXSIZE 64

// 9600 bit/s              // Geschwindigkeit ist whlbar
#define bittime_15 391   // Bitdauer fr 1,5 Sendebits in ticks a 400ns
#define bittime_10 260   // Bitdauer fr 1 Sendebit in ticks a 400ns

// 19200 bit/s             // Geschwindigkeit ist whlbar
//#define bittime_15 195   // Bitdauer fr 1,5 Sendebits in ticks a 400ns
//#define bittime_10 130   // Bitdauer fr 1 Sendebit in ticks a 400ns

// 38400 bit/s             // zu schnell, Senden geht, Empfangen macht Fehler
//#define bittime_15 98    // Bitdauer fr 1,5 Sendebits in ticks a 400ns
//#define bittime_10 65    // Bitdauer fr 1 Sendebit in ticks a 400ns

// 57600 bit/s             // zu schnell, Senden geht, Empfangen macht grobe Fehler
//#define bittime_15 65    // Bitdauer fr 1,5 Sendebits in ticks a 400ns
//#define bittime_10 43    // Bitdauer fr 1 Sendebit in ticks a 400ns

// 115200 bit/s            // zu schnell, Senden und Empfangen machen grobe Fehler
//#define bittime_15 33    // Bitdauer fr 1,5 Sendebits in ticks a 400ns
//#define bittime_10 22    // Bitdauer fr 1 Sendebit in ticks a 400ns


#define send_polling_intervall 10  // Abstand in dem Rinpuffer auf Sendebytes gepollt wird in Sendebitdauern

sbit P3_8 =P3^8;
sbit P3_9 =P3^9;
sbit P8_0 =P8^0;

sbit DP3_8 =DP3^8; // Receive von COM
sbit DP3_9 =DP3^9; // Transmit an COM
sbit DP8_0 =DP8^0; // umgeleitetes Receive Signal

// Structures ----------------------------------------------------------

struct buffer {
	unsigned char data [MAXSIZE];
	int p_in;
	int p_out;
};


// Globale Variablen ----------------------------------------------------------

struct buffer receive_buffer;
struct buffer send_buffer;


// Function Prototypes ----------------------------------------------------------

void send_receive_init (void);
void receive_isr (void);
void send_isr (void);
void buffer_put_nowait (unsigned char zeichen, struct buffer *buf);
char buffer_get_nowait (struct buffer *buffer);
void buffer_put_wait (unsigned char zeichen, struct buffer *buf);
char buffer_get_wait (struct buffer *buffer);


// Functions -------------------------------------------------------------------

void send_receive_init (void) {
 	CCM4=0x0002;  // Capture on Negativ Transition, Timer 7
	CC16IC=0x004C;	// CC16 fr receive_isr, IE=1 ILVL=0011  GLVL=00, Int-Nr. 0x30
	T78CON=0;
	T7R=1;	

  CC24=T7+send_polling_intervall*bittime_10;
  CCM6=0x0004;    // Compare Mode 0, Interrutp only, several interrupts per timer period, Timer 7
	CC24IC=0x004D;	// CC24 fr send_isr, IE=1 ILVL=0011  GLVL=01, Int-Nr. 0x38

  DP3_9=1;        // Sendepin auf Output stellen
}


void receive_isr (void) interrupt 0x30	
{
static int bit_counter_receive=0;
static unsigned char receive_byte=0;
static bit receive_bit;

  receive_bit=P8_0;
  switch(bit_counter_receive){
		case 0:
			CCM4 = 0x04;	
			CC16 += bittime_15;	
			bit_counter_receive++;
			break;
		case 1:
		case 2:
		case 3:
		case 4:
		case 5:
		case 6:
		case 7:	 	
			CC16 += bittime_10;	
			receive_byte = (receive_byte>>1)|(receive_bit?0x80:0);
			bit_counter_receive++;
			break;
		case 8:
			receive_byte = (receive_byte>>1)|(receive_bit?0x80:0);
			bit_counter_receive=0;
			CCM4 = 0x02;
			// in den Ringbuffer schreiben
			buffer_put_nowait(receive_byte, &receive_buffer);
			break;
    default: break;
  }
}


void send_isr (void) interrupt 0x38
{
static int bit_counter_send = 0;  // 0 Ruhezustand oder Stopbit luft, 1 Startbit luft, 
                                  // 2..9 D0 bis D7 laufen
static unsigned char send_byte;

  switch(bit_counter_send){
		case 0:
      if (send_buffer.p_in!=send_buffer.p_out){   // wenn im Sendepuffer was drin ist
        send_byte=buffer_get_nowait(&send_buffer);       // rausholen
        P3_9=0;                                   // Startbit ausgeben
        bit_counter_send=1;                       // bitcounter auf "Startbit luft"
        CC24+=bittime_10;                         // nchster Int in einer Bitdauer
      }
      else
			  CC24+=send_polling_intervall*bittime_10;  // nchstes Poll-Intervall aufziehen
			break;
		case 1:
		case 2:
		case 3:
		case 4:
		case 5:
		case 6:
		case 7:	 	
		case 8:    // 8 Datenbits werden gesendet
      P3_9=send_byte&0x01;      // niederwertigstes bit senden
      send_byte>>=1;            // nchstes Senden in send_byte vorbereiten durch rechts shift
      bit_counter_send++;       // bitcounter inkremtieren
      CC24+=bittime_10;         // nchster Int in einer Bitdauer
			break;
		case 9:    // letztes Datenbit wurde gesendet, Stopbit senden
      P3_9=1;                   // Stopbit senden
      bit_counter_send=0;       // bitcounter zurcksetzen
      CC24+=bittime_10;         // nchster Int in einer Bitdauer, knnte ja gleich noch ein Byte zu senden sein
			break;
    default: break;
  }
}



// stellt ein Byte in den Buffer, pat das Zeichen nicht mehr in den Buffer so wird es verworfen
void buffer_put_nowait (unsigned char zeichen, struct buffer *buf)
{
	if (((buf->p_in)-(buf->p_out)&0x003F)<MAXSIZE-1)
	{
		buf->data[buf->p_in]=zeichen;
		buf->p_in=(buf->p_in+1)&0x003F;
	}
}


// liest nicht wartend ein Byte aus Buffer, ist der Buffer leer so wird \0 zurckgegeben
char buffer_get_nowait (struct buffer *pBuffer)
{
char cRead='0';
	if (pBuffer->p_in != pBuffer->p_out)
	{
		cRead = pBuffer->data[pBuffer->p_out];
		pBuffer->p_out++;
		pBuffer->p_out = pBuffer->p_out & 0x003F;
		return cRead;
	}// if (pBuffer->p_in != pBuffer->p_out)
	else
	{// Fehler oder nichts zum Lesen
		return '\0';
	}
}



// stellt ein Byte in den Buffer, pat das Zeichen nicht mehr in den Buffer so wird gewartet
// bis die Sende-ISR wieder Platz geschaffen hat
void buffer_put_wait (unsigned char zeichen, struct buffer *buf)
{
	while (((buf->p_in)-(buf->p_out)&0x003F)>=(MAXSIZE-1));
	
	buf->data[buf->p_in]=zeichen;
	buf->p_in=(buf->p_in+1)&0x003F;
}


// liest wartend ein Byte aus Buffer, ist der Buffer leer so wird \0 zurckgegeben
char buffer_get_wait (struct buffer *pBuffer)
{
char cRead='0';

  while(pBuffer->p_in == pBuffer->p_out);

	cRead = pBuffer->data[pBuffer->p_out];
	pBuffer->p_out++;
	pBuffer->p_out = pBuffer->p_out & 0x003F;
	return cRead;
}



// Definiert man diese Funktion im eigenen Projekt so berschreibt sie die Funktion
// putchar in der Keil-Library. Die Ausgabe von putchar, printf und puts
// wird dann auf die SoftR232 umgeleitet.
// Ein einzelnes LF (aus \n am Ende von printf) oder
// CR (aus Enter nach Eingabe von Tastatur im Terminalprogramm MTTTY) am Zeilenende
// wird automatisch auf ein CR+LF erweitert (ntig fr zB Windows Notepad).
signed char putchar (signed char c)  {
  if (c==10) buffer_put_wait (13, &send_buffer); // automatisch aus LF ein CR+LF machen
  buffer_put_wait (c, &send_buffer);
  if (c==13) buffer_put_wait (10, &send_buffer); // automatisch aus LF ein CR+LF machen
  return (c);
}
